@@ -14,6 +14,8 @@
 #include <dirent.h>
 #include <utmp.h>
 #include <mntent.h>
+#include <sys/sysinfo.h>
+#include <sched.h>
 
 #include <Rinternals.h>
 
@@ -766,7 +768,7 @@ SEXP psll_memory_info(SEXP p) {
 
   *(buf + ret - 1) = '\0';
 
-  ret = sscanf(buf, "%lu %lu %lu %lu %lu %lu %lu", &rss, &vms, &shared,
+  ret = sscanf(buf, "%lu %lu %lu %lu %lu %lu %lu", &vms, &rss, &shared,
 	       &text, &lib, &data, &dirty);
   if (ret != 7) {
     ps__set_error_from_errno();
@@ -775,14 +777,22 @@ SEXP psll_memory_info(SEXP p) {
 
   PS__CHECK_HANDLE(handle);
 
-  PROTECT(result = allocVector(INTSXP, 7));
-  INTEGER(result)[0] = rss;
-  INTEGER(result)[1] = vms;
-  INTEGER(result)[2] = shared;
-  INTEGER(result)[3] = text;
-  INTEGER(result)[4] = lib;
-  INTEGER(result)[5] = data;
-  INTEGER(result)[6] = dirty;
+#ifdef _SC_PAGESIZE
+    // recommended POSIX
+    int page_size = sysconf(_SC_PAGESIZE);
+#elif _SC_PAGE_SIZE
+    // alias
+    int page_size = sysconf(_SC_PAGE_SIZE);
+#endif
+
+  PROTECT(result = allocVector(REALSXP, 7));
+  REAL(result)[0] = rss * page_size;
+  REAL(result)[1] = vms * page_size;
+  REAL(result)[2] = shared * page_size;
+  REAL(result)[3] = text * page_size;
+  REAL(result)[4] = lib * page_size;
+  REAL(result)[5] = data * page_size;
+  REAL(result)[6] = dirty * page_size;
   PROTECT(names = ps__build_string("rss", "vms", "shared", "text", "lib",
 				   "data", "dirty", NULL));
   setAttrib(result, R_NamesSymbol, names);
@@ -1074,6 +1084,131 @@ SEXP psll_connections(SEXP p) {
   return result;
 }
 
+SEXP ps__memory_maps(SEXP p) {
+  ps_handle_t *handle = R_ExternalPtrAddr(p);
+  char path[512];
+  int ret;
+  char *buf;
+
+  if (!handle) error("Process pointer cleaned up already");
+
+  ret = snprintf(path, sizeof(path), "/proc/%i/smaps", handle->pid);
+  if (ret >= sizeof(path)) {
+    ps__set_error("Cannot read proc, path buffer too small");
+    ps__throw_error();
+  } else if (ret < 0) {
+    ps__set_error_from_errno();
+    ps__throw_error();
+  }
+
+  ret = ps__read_file(path, &buf, /* buffer= */ 2048);
+  if (ret == -1) {
+    ps__wrap_linux_error(handle);
+    ps__throw_error();
+  }
+
+  ps__check_for_zombie(handle, 0);
+
+  SEXP result = PROTECT(allocVector(STRSXP, 1));
+  SET_STRING_ELT(result, 0, Rf_mkCharLenCE(buf, ret, CE_UTF8));
+  UNPROTECT(1);
+  return result;
+}
+
+static const int NCPUS_START = sizeof(unsigned long) * CHAR_BIT;
+
+SEXP psll_get_cpu_aff(SEXP p) {
+  ps_handle_t *handle = R_ExternalPtrAddr(p);
+  int cpu, ncpus, count, cpucount_s;
+  pid_t pid;
+  size_t setsize;
+  cpu_set_t *mask = NULL;
+
+  if (!handle) error("Process pointer cleaned up already");
+
+  PS__CHECK_HANDLE(handle);
+
+  ncpus = NCPUS_START;
+  while (1) {
+    setsize = CPU_ALLOC_SIZE(ncpus);
+    mask = CPU_ALLOC(ncpus);
+    if (mask == NULL) {
+      ps__no_memory("");
+      goto error;
+    }
+    if (sched_getaffinity(pid, setsize, mask) == 0) {
+      break;
+    }
+    CPU_FREE(mask);
+    mask = NULL;
+    if (errno != EINVAL) {
+      ps__set_error_from_errno();
+      goto error;
+    }
+    if (ncpus > INT_MAX / 2) {
+      ps__set_error("could not allocate a large enough CPU set");
+      goto error;
+    }
+    ncpus = ncpus * 2;
+  }
+
+  cpucount_s = CPU_COUNT_S(setsize, mask);
+
+  /* This is unsafe, in that in case of an R exception, CPU_FREE will
+     not be called. But it is also very hard to fix without cleancall. */
+  PROTECT_INDEX pidx;
+  SEXP result;
+  PROTECT_WITH_INDEX(result = allocVector(INTSXP, cpucount_s), &pidx);
+  int pp = 0;
+
+  for (cpu = 0, count = cpucount_s; count; cpu++) {
+    if (CPU_ISSET_S(cpu, setsize, mask)) {
+      INTEGER(result)[pp++] = cpu;
+      --count;
+    }
+  }
+  CPU_FREE(mask);
+
+  REPROTECT(result = Rf_lengthgets(result, pp), pidx);
+  UNPROTECT(1);
+  return result;
+
+error:
+  if (mask) CPU_FREE(mask);
+  ps__throw_error();
+  return R_NilValue;
+}
+
+SEXP psll_set_cpu_aff(SEXP p, SEXP affinity) {
+  ps_handle_t *handle = R_ExternalPtrAddr(p);
+  cpu_set_t cpu_set;
+  pid_t pid;
+  int i, seq_len = LENGTH(affinity);
+
+  if (!handle) error("Process pointer cleaned up already");
+
+  pid = handle->pid;
+
+  CPU_ZERO(&cpu_set);
+  for (i = 0; i < seq_len; i++) {
+    int value = INTEGER(affinity)[i];
+    CPU_SET(value, &cpu_set);
+  }
+
+  PS__CHECK_HANDLE(handle);
+
+  if (sched_setaffinity(pid, sizeof(cpu_set), &cpu_set)) {
+    ps__set_error_from_errno();
+    goto error;
+  }
+
+  return R_NilValue;
+
+error:
+  ps__throw_error();
+  return R_NilValue;
+}
+
 SEXP ps__users() {
   struct utmp *ut;
   SEXP result;
@@ -1142,9 +1277,38 @@ SEXP ps__disk_partitions(SEXP all) {
 error:
   if (file != NULL) endmntent(file);
   ps__throw_error();
+  /* These are never called, but R CMD check and rchk cannot handle this */
+  error("nah");
   return R_NilValue;
 }
 
+SEXP ps__loadavg(SEXP counter_name) {
+  /* Try /proc first, if fails we try sysinfo() */
+  struct sysinfo info;
+  char *buf;
+  int ret = ps__read_file("/proc/loadavg", &buf, 128);
+  if (ret == -1) goto sysinfo;
+
+  SEXP avg = PROTECT(allocVector(REALSXP, 3));
+  if (sscanf(buf, "%lf %lf %lf", REAL(avg), REAL(avg) + 1, REAL(avg) + 2) == 3) {
+    UNPROTECT(1);
+    return avg;
+  }
+
+ sysinfo:
+  if (sysinfo(&info) < 0) {
+    ps__set_error_from_errno();
+    ps__throw_error();
+  }
+
+  REAL(avg)[0] = (double) info.loads[0] / 65536.0;
+  REAL(avg)[1] = (double) info.loads[1] / 65536.0;
+  REAL(avg)[2] = (double) info.loads[2] / 65536.0;
+
+  UNPROTECT(1);
+  return avg;
+}
+
 SEXP ps__system_memory() {
   // This is implemented in R on Linux
   ps__throw_error();
@@ -1155,3 +1319,9 @@ SEXP ps__system_swap() {
   // TODO
   return R_NilValue;
 }
+
+SEXP ps__system_cpu_times() {
+  // This is implemented in R on Linux
+  ps__throw_error();
+  return R_NilValue;
+}
