@@ -862,6 +862,36 @@ SEXP psll_open_files(SEXP p) {
   return result;
 }
 
+SEXP psll_dlls(SEXP p) {
+  ps_handle_t *handle = R_ExternalPtrAddr(p);
+  HANDLE processHandle = NULL;
+  DWORD access = PROCESS_QUERY_INFORMATION | PROCESS_VM_READ;
+  SEXP result;
+
+  if (!handle) error("Process pointer cleaned up already");
+
+  processHandle = ps__handle_from_pid_waccess(handle->pid, access);
+  if (processHandle == NULL) {
+    PS__CHECK_HANDLE(handle);
+    ps__set_error_from_windows_error(0);
+    ps__throw_error();
+  }
+
+  PROTECT(result = ps__get_modules(processHandle));
+
+  CloseHandle(processHandle);
+
+  PS__CHECK_HANDLE(handle);
+
+  if (isNull(result)) {
+    ps__set_error_from_windows_error(0);
+    ps__throw_error();
+  }
+
+  UNPROTECT(1);
+  return result;
+}
+
 SEXP psll_interrupt(SEXP p, SEXP ctrlc, SEXP interrupt_path) {
   ps_handle_t *handle = R_ExternalPtrAddr(p);
   const char *cinterrupt_path = CHAR(STRING_ELT(interrupt_path, 0));
@@ -936,6 +966,123 @@ SEXP psll_interrupt(SEXP p, SEXP ctrlc, SEXP interrupt_path) {
   return R_NilValue;
 }
 
+static int ps_get_proc_wset_information(SEXP p, HANDLE hProcess,
+					PMEMORY_WORKING_SET_INFORMATION *wSetInfo) {
+
+  ps_handle_t *handle = R_ExternalPtrAddr(p);
+  if (!handle) error("Process pointer cleaned up already");
+
+  DWORD pid = handle->pid;
+  NTSTATUS status;
+  PVOID buffer;
+  SIZE_T bufferSize;
+
+  bufferSize = 0x8000;
+  buffer = MALLOC_ZERO(bufferSize);
+  if (! buffer) {
+    ps__no_memory("get wset information");
+    ps__throw_error();
+  }
+
+  while (1) {
+    status = NtQueryVirtualMemory(
+      hProcess,
+      NULL,
+      MemoryWorkingSetInformation,
+      buffer,
+      bufferSize,
+      NULL
+    );
+    if (status != STATUS_INFO_LENGTH_MISMATCH) {
+      break;
+    }
+
+    FREE(buffer);
+    bufferSize *= 2;
+    // Fail if we're resizing the buffer to something very large.
+    if (bufferSize > 256 * 1024 * 1024) {
+      ps__set_error("NtQueryVirtualMemory bufsize is too large");
+      ps__throw_error();
+    }
+    buffer = MALLOC_ZERO(bufferSize);
+    if (! buffer) {
+      ps__no_memory("NtQueryVirtualMemory");
+      ps__throw_error();
+    }
+  }
+
+  if (!NT_SUCCESS(status)) {
+    if (status == STATUS_ACCESS_DENIED) {
+      ps__access_denied("NtQueryVirtualMemory -> STATUS_ACCESS_DENIED");
+    } else if (!LOGICAL(ps__is_running(handle))[0]) {
+      ps__no_such_process(pid, "NtQueryVirtualMemory");
+    } else {
+      ps__set_error_from_windows_error(0);
+    }
+    HeapFree(GetProcessHeap(), 0, buffer);
+    ps__throw_error();
+  }
+
+  *wSetInfo = (PMEMORY_WORKING_SET_INFORMATION) buffer;
+  return 0;
+}
+
+/*
+ * Returns the USS of the process.
+ * Reference:
+ * https://dxr.mozilla.org/mozilla-central/source/xpcom/base/
+ *     nsMemoryReporterManager.cpp
+ */
+
+SEXP psll_memory_uss(SEXP p) {
+  ps_handle_t *handle = R_ExternalPtrAddr(p);
+  if (!handle) error("Process pointer cleaned up already");
+
+  DWORD pid = handle->pid;
+  HANDLE hProcess = ps__handle_from_pid(pid);
+  PS__PROCESS_WS_COUNTERS wsCounters;
+  PMEMORY_WORKING_SET_INFORMATION wsInfo;
+  ULONG_PTR i;
+
+  if (! hProcess) {
+    ps__throw_error();
+  }
+
+  if (ps_get_proc_wset_information(p, hProcess, &wsInfo) != 0) {
+    CloseHandle(hProcess);
+    return NULL;
+  }
+  memset(&wsCounters, 0, sizeof(PS__PROCESS_WS_COUNTERS));
+
+  for (i = 0; i < wsInfo->NumberOfEntries; i++) {
+    // This is what ProcessHacker does.
+    /*
+      wsCounters.NumberOfPages++;
+      if (wsInfo->WorkingSetInfo[i].ShareCount > 1)
+      wsCounters.NumberOfSharedPages++;
+      if (wsInfo->WorkingSetInfo[i].ShareCount == 0)
+      wsCounters.NumberOfPrivatePages++;
+      if (wsInfo->WorkingSetInfo[i].Shared)
+      wsCounters.NumberOfShareablePages++;
+    */
+
+    // This is what we do: count shared pages that only one process
+    // is using as private (USS).
+    if (!wsInfo->WorkingSetInfo[i].Shared ||
+	wsInfo->WorkingSetInfo[i].ShareCount <= 1) {
+      wsCounters.NumberOfPrivatePages++;
+    }
+  }
+
+  HeapFree(GetProcessHeap(), 0, wsInfo);
+  CloseHandle(hProcess);
+
+  SYSTEM_INFO sysinfo;
+  GetNativeSystemInfo(&sysinfo);
+
+  return Rf_ScalarInteger(wsCounters.NumberOfPrivatePages * sysinfo.dwPageSize);
+}
+
 SEXP ps__users() {
   HANDLE hServer = WTS_CURRENT_SERVER_HANDLE;
   WCHAR *buffer_user = NULL;
@@ -1366,3 +1513,135 @@ error:
   return R_NilValue;
 
 }
+
+
+SEXP psll_get_cpu_aff(SEXP p) {
+  ps_handle_t *handle = R_ExternalPtrAddr(p);
+  HANDLE hProcess = NULL;
+  DWORD access = PROCESS_QUERY_LIMITED_INFORMATION;
+  DWORD_PTR proc_mask;
+  DWORD_PTR system_mask;
+
+  if (!handle) error("Process pointer cleaned up already");
+
+  hProcess = ps__handle_from_pid_waccess(handle->pid, access);
+  if (!hProcess) {
+    PS__CHECK_HANDLE(handle);
+    ps__set_error_from_windows_error(0);
+    goto error;
+  }
+
+  BOOL ret = GetProcessAffinityMask(hProcess, &proc_mask, &system_mask);
+  if (!ret) {
+    ps__set_error_from_windows_error(0);
+    goto error;
+  }
+
+  CloseHandle(hProcess);
+
+  unsigned long long n = (unsigned long long) proc_mask;
+  int count = 0, w = 0;
+  while (n) {
+    count += n & 1;
+    n >>= 1;
+  }
+  SEXP result = PROTECT(Rf_allocVector(INTSXP, count));
+  n = (unsigned long long) proc_mask;
+  count = 0;
+  while (n) {
+    if (n & 1) {
+      INTEGER(result)[count++] = w;
+    }
+    n >>= 1;
+    w++;
+  }
+
+  UNPROTECT(1);
+  return result;
+
+error:
+  if (hProcess) CloseHandle(hProcess);
+  ps__throw_error();
+  return R_NilValue;
+}
+
+SEXP psll_set_cpu_aff(SEXP p, SEXP affinity) {
+  ps_handle_t *handle = R_ExternalPtrAddr(p);
+  HANDLE hProcess = NULL;
+  DWORD access = PROCESS_QUERY_INFORMATION | PROCESS_SET_INFORMATION;
+  DWORD_PTR mask = 0;
+
+  if (!handle) error("Process pointer cleaned up already");
+
+  hProcess = ps__handle_from_pid_waccess(handle->pid, access);
+  if (!hProcess) {
+    PS__CHECK_HANDLE(handle);
+    ps__set_error_from_windows_error(0);
+    goto error;
+  }
+
+  int *caff = INTEGER(affinity);
+  int i, len = LENGTH(affinity);
+  for (i = 0; i < len; i++) {
+    int m = 1 << caff[i];
+    mask |= m;
+  }
+
+  BOOL ret = SetProcessAffinityMask(hProcess, mask);
+  if (!ret) {
+    PS__CHECK_HANDLE(handle);
+    ps__set_error_from_windows_error(0);
+    goto error;
+  }
+
+  CloseHandle(hProcess);
+
+  return R_NilValue;
+
+error:
+  if (hProcess) CloseHandle(hProcess);
+  ps__throw_error();
+  return R_NilValue;
+}
+
+SEXP ps__loadavg(SEXP counter_name) {
+  SEXP ret = PROTECT(allocVector(REALSXP, 3));
+  ps__get_loadavg(REAL(ret), counter_name);
+  UNPROTECT(1);
+  return ret;
+}
+
+#define LO_T 1e-7
+#define HI_T 429.4967296
+
+SEXP ps__system_cpu_times() {
+  double idle, kernel, user, system;
+  FILETIME idle_time, kernel_time, user_time;
+
+  if (!GetSystemTimes(&idle_time, &kernel_time, &user_time)) {
+    ps__set_error_from_windows_error(0);
+    ps__throw_error();
+  }
+
+  idle = (double)((HI_T * idle_time.dwHighDateTime) +
+		  (LO_T * idle_time.dwLowDateTime));
+  user = (double)((HI_T * user_time.dwHighDateTime) +
+		  (LO_T * user_time.dwLowDateTime));
+  kernel = (double)((HI_T * kernel_time.dwHighDateTime) +
+		    (LO_T * kernel_time.dwLowDateTime));
+
+  // Kernel time includes idle time.
+  // We return only busy kernel time subtracting idle time from
+  // kernel time.
+  system = (kernel - idle);
+
+  const char *nms[] = { "user", "system", "idle", "" };
+  SEXP ret = PROTECT(Rf_mkNamed(REALSXP, nms));
+
+  REAL(ret)[0] = (double) user;
+  REAL(ret)[1] = (double) system;
+  REAL(ret)[2] = (double) idle;
+
+  UNPROTECT(1);
+  return ret;
+}
