@@ -1,6 +1,4 @@
 
-context("common")
-
 test_that("create self process", {
   expect_error(ps_handle("foobar"), class = "invalid_argument")
   expect_error(ps_handle(time = 123), class = "invalid_argument")
@@ -155,7 +153,7 @@ test_that("cwd", {
   ps <- ps_handle(p1$get_pid())
   expect_true(ps_is_running(ps))
 
-  expect_equal(ps_cwd(ps), normalizePath(tempdir()))
+  expect_equal(normalizePath(ps_cwd(ps)), normalizePath(tempdir()))
 })
 
 test_that("environ, environ_raw", {
@@ -309,3 +307,19 @@ test_that("interrupt", {
   expect_false(ps_is_running(ps))
   if (ps_os_type()[["POSIX"]]) expect_equal(px$get_exit_status(), -2)
 })
+
+test_that("cpu affinity", {
+  skip_on_cran()
+  skip_on_covr()
+  skip_on_os("mac")
+
+  orig <- ps::ps_get_cpu_affinity()
+  expect_true(length(orig) <= ps::ps_cpu_count())
+
+  do <- function() {
+    ps::ps_set_cpu_affinity(affinity = 0:0)
+    ps::ps_get_cpu_affinity()
+  }
+
+  expect_equal(callr::r(do), 0:0)
+})
