@@ -1,6 +1,4 @@
 
-context("system")
-
 test_that("ps_pids", {
   pp <- ps_pids()
   expect_true(is.integer(pp))
@@ -56,3 +54,11 @@ test_that("ps_os_name", {
 test_that("ps_users runs", {
   expect_error(ps_users(), NA)
 })
+
+test_that("ps_cpu_count", {
+  log <- ps_cpu_count(logical = TRUE)
+  phy <- ps_cpu_count(logical = FALSE)
+  if (!is.na(log) && !is.na(phy)) expect_true(log >= phy)
+  if (!is.na(log)) expect_true(log > 0)
+  if (!is.na(phy)) expect_true(phy > 0)
+})
